/******************************************************************************
 *  Name:        Jeremie Lumbroso
 *  NetID:       lumbroso
 *  Precept:     L01
 * 
 *  Description: Provides a method to create a Polynomial object containing
 *               a single term.
 *               
 *               Provides a method to convert any Polynomial object into a
 *               string, by using only the coeff(int expo) method of the
 *               class.
 *
 *               Using this functionality, this class also provides short
 *               test methods to test whether a Polynomial object's string
 *               representation matches some predetermined value. (This is
 *               useful for unit testing of the class.)
 ******************************************************************************/

public class PolynomialTools {
    
    
    // Tolerance with which to make comparisons for runEvaluateTest
    
    private static final double DOUBLE_COMPARISON_TOLERANCE = 0.0001;
    
    // Private setting for whether to display "*x^0" for the terms of
    // exponent 0
    
    private static boolean DISPLAY_X0 = false;
    
    
    // Private setting for whether to display warnings even when
    // assertions are turned off.
    
    private static boolean OVERRIDE_DISABLED_ASSERTION = true;
    
    
    public static Polynomial makeSingleTerm(double c, int expo) {
        // create an array that has only a single entry
        double[] p_arr = new double[expo+1];
        
        // fill the last entry with the coefficient (all other are equal 0.0)
        p_arr[expo] = c;
        
        // we can now use the Polynomial constructor
        Polynomial p_poly = new Polynomial(p_arr);
        
        return p_poly;
    }
    
    
    // Takes a Polynomial and returns a string representation of it
    
    public static String toString(Polynomial obj) {

        // StringBuilder is more appropriate when constructing strings
        
        StringBuilder sb = new StringBuilder();
        
        boolean first = true;
        
        for(int i = 0; i < Polynomial.MAX_EXPONENT; i++) {
            double coeff = obj.coeff(i);
            
            // is this a non-zero coefficient?
            
            // SIDENOTE: comparison of doubles is generally not acceptable
            // because of rounding errors; however what we are doing here
            // is checking whether the coefficient is equal to the initial
            // value 0.0 (suggesting the value comes from a "return 0.0"
            // that follows from a coefficient not existing).
            
            if(coeff != 0.0) {
                if (!first)
                    sb.append(" + ");
                
                if (i == 0) {
                    if (DISPLAY_X0)
                        sb.append(coeff + "*x^0");
                    else
                        sb.append(coeff);
                } else
                    sb.append(coeff + "*x^" + i);
                              
                first = false;
            }
        }

        // special case of zero polynomial: normally it will be computed as
        // the empty string (empty list of non-zero coefficients)

        String result = sb.toString();
        
        if(result.length() == 0)
            result = "0.0";
        
        return result;
    }
    
    
    // Takes a Polynomial 'p' and String 's' and checks whether the string
    // representation of 'p' is equal to 's'
    //
    // NOTE: by default this method use assertions, but if these are not
    // enabled, it will print to the standard output in case of error.
    
    public static void runStringTest(Polynomial p, String s) {        
        // convert Polynomial 'p' to String
        String p_str = PolynomialTools.toString(p);
        
        // create a String to display the test to a human being
        String test_caption = p_str + " (obtained) != " + s + " (desired)";
        
        // check if the resulting string is equal to the provided string 's'
        boolean test_success = (p_str.compareTo(s) == 0);
        
        // check if it is equal to the provided string 's'
        assert test_success : "CORRECTNESS failed: " + test_caption;
        
        // detect if assertions are off and raise warning message anyway
        if (OVERRIDE_DISABLED_ASSERTION) {
            boolean assert_on = false;
            assert (assert_on = true);
            if (!assert_on && !test_success) {
                System.out.print("WARNING: assertions not enabled; ");
                System.out.println("CORRECTNESS test would have failed: "
                                   + test_caption);
            }
        }
    }
    
    
    // Takes a Polynomial 'p', a double 'x' and a reference value 'ref'
    // and checks whether 'p', evaluated in 'x' is equal to 'ref'.
    //
    // NOTE: for more information, read the following after the exam:
    //
    //     http://bit.ly/PolynomialIdentityTestingPDF
    
    public static void runEvaluateTest(Polynomial p, double x, double ref) {        
        // evaluate the polynomial 'p' in 'x'
        double p_val = p.evaluate(x);
        
        // create a String to display the test to a human being
        String test_caption = p_val + " (obtained) != " + ref + " (desired)";
        
        // check if the resulting value is within a small tolerance of
        // reference value
        boolean test_success =
            Math.abs(p_val - ref) <= DOUBLE_COMPARISON_TOLERANCE;
        
        // check if it is equal to the provided string 's'
        assert test_success : "evaluate test failed: " + test_caption;
        
        // detect if assertions are off and raise warning message anyway
        if (OVERRIDE_DISABLED_ASSERTION) {
            boolean assert_on = false;
            assert (assert_on = true);
            if (!assert_on && !test_success) {
                System.out.print("WARNING: assertions not enabled; ");
                System.out.println("evaluate test would have failed: "
                                   + test_caption);
            }
        }
    }

}
